@@ -22,7 +22,7 @@
  *  `LICENSE' that comes with the fcron source distribution.
  */
 
- /* $Id: conf.c,v 1.66 2003/12/25 22:43:57 thib Exp thib $ */
+ /* $Id: conf.c,v 1.68 2004/08/12 09:43:41 thib Exp thib $ */
 
 #include "fcron.h"
 
@@ -437,11 +437,10 @@ read_file(const char *file_name, cf_t *cf)
     struct passwd *pass = NULL;
     short int type = 0, size = 0;
     int rc;
-#ifdef CONFIG_FLASK
-    int flask_enabled = is_flask_enabled();
-    struct security_query qry;
-    struct security_response rsp;
+#ifdef WITH_SELINUX
+    int flask_enabled = is_selinux_enabled();
     int retval;
+    struct av_decision avd;
     const char *user_name;
 #endif
 
@@ -453,16 +452,18 @@ read_file(const char *file_name, cf_t *cf)
 
     /* check if this file is owned by root : otherwise, all runas fields
      * of this field should be set to the owner */
-#ifdef CONFIG_FLASK
-    if(flask_enabled)
-	rc = fstat_secure(fileno(ff), &file_stat, &cf->cf_file_sid);
-    else
-#endif
     rc = fstat(fileno(ff), &file_stat);
     if ( rc != 0 ) {
 	error_e("Could not stat %s", file_name);
 	goto err;
     }
+#ifdef WITH_SELINUX
+    if(flask_enabled && fgetfilecon(fileno(ff), &cf->cf_file_context) < 0 )
+    {
+	error_e("Could not get context of %s", file_name);
+	goto err;
+    }
+#endif
 
     if ( strncmp(file_name,"new.", 4) == 0 ) {
 	if ( file_stat.st_uid == ROOTUID ) {
@@ -495,7 +496,7 @@ read_file(const char *file_name, cf_t *cf)
 	}
     }
 
-#ifdef CONFIG_FLASK
+#ifdef WITH_SELINUX
     /*
      * Since crontab files are not directly executed,
      * crond must ensure that the crontab file has
@@ -507,17 +508,20 @@ read_file(const char *file_name, cf_t *cf)
 	user_name = "system_u";
     else
 	user_name = cf->cf_user;
-    if(get_default_sid(user_name, 0, &cf->cf_user_sid))
-	error_e("NO SID for user \"%s\"", cf->cf_user_sid);
-    qry.ssid = cf->cf_user_sid;
-    qry.tsid = cf->cf_file_sid;
-    qry.tclass = SECCLASS_FILE;
-    qry.requested = FILE__ENTRYPOINT;
-    retval = security_compute_av(&qry, &rsp);
-    if(retval || ((qry.requested & rsp.allowed) != qry.requested)) {
-	syslog(LOG_ERR, "ENTRYPOINT FAILED for \"%s\" (SID %u) for file SID %u"
-	       , cf->cf_user, cf->cf_user_sid, cf->cf_file_sid);
-	goto err;
+    if(flask_enabled)
+    {
+	if(get_default_context(user_name, NULL, &cf->cf_user_context))
+	    error_e("NO CONTEXT for user \"%s\"", cf->cf_user_context);
+	retval = security_compute_av(cf->cf_user_context, cf->cf_file_context
+		, SECCLASS_FILE, FILE__ENTRYPOINT, &avd);
+
+	if(retval || ((FILE__ENTRYPOINT & avd.allowed) != FILE__ENTRYPOINT))
+	{
+	    syslog(LOG_ERR, "ENTRYPOINT FAILED for user \"%s\" "
+		   "(CONTEXT %s) for file CONTEXT %s"
+		   , cf->cf_user, cf->cf_user_context, cf->cf_file_context);
+	    goto err;
+	}
     }
 #endif
 
@@ -525,7 +529,7 @@ read_file(const char *file_name, cf_t *cf)
     bzero(buf, sizeof(buf));
 
     /* get version of fcrontab file: it permits to daemon not to load
-     * a file which he won't understand the syntax, for exemple
+     * a file which he won't understand the syntax, for example
      * a file using a depreciated format generated by an old fcrontab,
      * if the syntax has changed */
     if ( read_type(fileno(ff), &type, &size) != OK || type != S_HEADER_T ||
@@ -775,18 +779,18 @@ add_line_to_file(cl_t *cl, cf_t *cf, uid_t runas, char *runas_str, time_t t_save
 	cl->cl_numexe = 0;
 	if ( is_lavg(cl->cl_option) ) {
 	    if ( ! is_strict(cl->cl_option) )
-		add_lavg_job(cl);
+		add_lavg_job(cl, -1);
 	}
 	else if ( is_serial(cl->cl_option) 
 		  || is_serial_once(cl->cl_option) )
-	    add_serial_job(cl);
+	    add_serial_job(cl, -1);
 	else {
 	    /* job has been stopped during execution :
 	     * launch it again */
 	    warn("job %s did not finish : running it again.",
 		 cl->cl_shell);
 	    set_serial_once(cl->cl_option);
-	    add_serial_job(cl);
+	    add_serial_job(cl, -1);
 	}
     }
 
@@ -796,7 +800,7 @@ add_line_to_file(cl_t *cl, cf_t *cf, uid_t runas, char *runas_str, time_t t_save
 	if ( cl->cl_nextexe <= now ) {
 	    if ( cl->cl_nextexe == 0 )
 		/* the is a line from a new file */
-		set_next_exe(cl, NO_GOTO);		    
+		set_next_exe(cl, NO_GOTO, -1);
 	    else if (cl->cl_runfreq == 1 &&
 		     is_notice_notrun(cl->cl_option))
 		set_next_exe_notrun(cl, SYSDOWN);
@@ -811,12 +815,12 @@ add_line_to_file(cl_t *cl, cf_t *cf, uid_t runas, char *runas_str, time_t t_save
 		    debug("   boot-run %s", cl->cl_shell);
 		    if ( ! is_lavg(cl->cl_option) ) {
 			set_serial_once(cl->cl_option);
-			add_serial_job(cl);
+			add_serial_job(cl, -1);
 		    }
 		    else
-			add_lavg_job(cl);			    
+			add_lavg_job(cl, -1);
 		}
-		set_next_exe(cl, STD);
+		set_next_exe(cl, STD, -1);
 	    }
 	    else {
 		if ( is_notice_notrun(cl->cl_option) ) {
@@ -824,11 +828,11 @@ add_line_to_file(cl_t *cl, cf_t *cf, uid_t runas, char *runas_str, time_t t_save
 		    struct tm *since2 = localtime(&cl->cl_nextexe);
 		    struct tm since;
 		    memcpy(&since, since2, sizeof(since));
-		    set_next_exe(cl, NO_GOTO);
+		    set_next_exe(cl, NO_GOTO, -1);
 		    mail_notrun(cl, SYSDOWN, &since);
 		} 
 		else
-		    set_next_exe(cl, NO_GOTO);
+		    set_next_exe(cl, NO_GOTO, -1);
 	    }
 	}
 	else
