@@ -22,7 +22,7 @@
  *  `LICENSE' that comes with the fcron source distribution.
  */
 
- /* $Id: fileconf.c,v 1.72 2003/12/29 20:00:40 thib Exp thib $ */
+ /* $Id: fileconf.c,v 1.74 2004/07/11 18:08:08 thib Exp thib $ */
 
 #include "fcrontab.h"
 
@@ -341,7 +341,10 @@ read_env(char *ptr, cf_t *cf)
 	Alloc(env, env_t);	
 
 	strncat(name, "=", sizeof(name) - strlen(name) - 1);
-	env->e_val = strdup2( strncat(name,val,sizeof(name)-strlen(name)-1) );
+	name[sizeof(name)-1]='\0';
+	strncat(name,val,sizeof(name)-strlen(name)-1);
+	name[sizeof(name)-1]='\0';
+	env->e_val = strdup2( name );
 	env->e_next = cf->cf_env_base;
 	cf->cf_env_base = env;
     }
@@ -433,7 +436,7 @@ read_opt(char *ptr, cl_t *cl)
     /* read one or several options and fill in the field "option" */
 {
     char opt_name[20];
-    unsigned int i;
+    int i;
     char in_brackets;
     
 #define Handle_err \
